/***************************************************************************
                             graphwidget.h
                             -------------------
    begin                : Fri Jun 10 2005
    copyright            : (C) 2005 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHWIDGET_H
#define GRAPHWIDGET_H

#include <qcanvas.h>
#include <qpopupmenu.h>
#include <qdict.h>
#include "graphviz.h"

class GraphNode;
class GraphEdge;

/**
 * A widget that displays call tree graphs generated by graphviz.
 * This class is based on a QCanvasView widget, and displays two types of
 * canvas items: GraphNode, which draws the name of a function inside a
 * polygon, and ArrowEdge, which is a directed graph edge shaped like an
 * arrow.
 * The call tree graph is populated using the addNode() method. The first
 * call creates a root node. Subsequent calls add nodes which represent either
 * functions called by a previously inserted node, or that are calling such
 * a node. A directed edge is created to depict the relationship between a
 * node and its parent.
 * Drawing is done through the embedded Agraph_t object (a graph, as defined
 * by the graphviz libraray). When the draw() method is called, the graph is
 * layed out in memory. The class then uses the CodeGenerator class to
 * obtain a set of instructions on how to actually draw the graph. These
 * instructions are used to create the appropriate canvas items.
 * An application _must_ call GraphWidget::init() before attempting to use
 * this class. It should also call GraphWidget::fini() when it no longer needs
 * any of these widgets.
 * @author Elad Lahav
 */
class GraphWidget : public QCanvasView
{
	Q_OBJECT
	
public:
	GraphWidget(QWidget* pParent = 0, const char* szName = 0);
	~GraphWidget();
	
	/**
	 * Information on a function call, as produced by a Cscope query.
	 * This structure is used for adding calls to the graph.
	 * @see addCall()
	 */
	struct CallData
	{
		/** The name of the calling function. */
		QString m_sCaller;
		
		/** The name of the called function. */
		QString m_sCallee;
		
		/** Path of the file in which the call appears. */
		QString m_sFile;
		
		/** The line number of the call. */
		QString m_sLine;
		
		/** The call's text. */
		QString m_sText;
	};

	/** Graph orientation values. */
	enum Orientation { Portrait, Landscape };

	void reset();
	void setRoot(const QString&);
	void addCall(const CallData&);
	void draw();
	bool load(const QString&);
	void save(const QString&, bool);
	void zoom(bool);
	void rotate();
	QString getTip(const QPoint&, QRect&);
	
	static void init();
	static void fini();
	
signals:
	void showCalled(const QString&);
	void showCalling(const QString&);
	void findDef(const QString&);
	void lineRequested(const QString&, uint);
	
protected:
	Agraph_t* getGraph() { return m_pGraph; }
	GVC_t* getContext() { return s_pGVC; }
	
	void resize(const QSize&);
	void startNode(Agnode_t*);
	void endNode();
	void startEdge(Agedge_t*);
	void endEdge();
	void polygon(const QPointArray&);
	void curve(const QPointArray&);
	void text(const QString&, const QPoint&);
	void setPen(const QColor& clr) { m_pen.setColor(clr); }
	void setBrush(const QColor& clr) { m_brush.setColor(clr); }
	void setFont(const QFont& font) { m_font = font; }
	void writeHeader(FILE*);
	
	virtual void contentsMousePressEvent(QMouseEvent*);
	
	friend class CodeGenerator;
	
private:
	/** The canvas object that controls this view. All canvas items are
		created as children of this object.	*/
	QCanvas* m_pCanvas;
	
	/** A graphviz graph object, used to lay out and draw the call tree
		graph. */
	Agraph_t* m_pGraph;
	
	/** The current pen to use for the canvas items. */
	QPen m_pen;
	
	/** The current brush to use for the canvas items. */
	QBrush m_brush;
	
	/** The current font to use for textual canvas items. */
	QFont m_font;
	
	/** The item currently being drawn. This item is set by the startNode()
		and startEdge() methods. */
	QCanvasPolygonalItem* m_pCurItem;
	
	/** The node over which the popup menu has been invoked. */
	QCanvasPolygonalItem* m_pMenuItem;
	
	/** A popup menu that appears when a node is right-clicked. */
	QPopupMenu* m_pNodePopup;
	
	/** A popup menu that appears when an edge is right-clicked. */
	QPopupMenu* m_pEdgePopup;

	/** The zoom factor for the graph. */
	double m_dZoom;
	
	void setNodeAttrib();
	inline void setCalledShown(GraphNode*, bool);
	inline void setCallingShown(GraphNode*, bool);
	inline bool getCalledShown(GraphNode*);
	inline bool getCallingShown(GraphNode*);
	void removeEdges(GraphNode*, bool);
	void showNodeMenu(GraphNode*, const QPoint&);
	void showEdgeMenu(GraphEdge*, const QPoint&);
	
	/** A graphviz rendering environment. */
	static GVC_t* s_pGVC;
	
private slots:
	void slotShowCalled();
	void slotShowCalling();
	void slotHideCalled();
	void slotHideCalling();
	void slotFindDef();
	void slotOpenCall();
};

#endif
